@@ -158,8 +158,8 @@ impl<B: AsRef<[u8]> + AsMut<[u8]> + Copy> Decode for CopyableBytesDecoder<B> {
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        false
+    fn is_idle(&self) -> bool {
+        self.offset == 0
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -227,6 +227,7 @@ impl<B: AsRef<[u8]> + AsMut<[u8]>> Decode for BytesDecoder<B> {
             size
         };
         if self.exact_requiring_bytes() == 0 {
+            self.offset = 0;
             Ok((size, self.bytes.take()))
         } else {
             track_assert!(!eos.is_reached(), ErrorKind::UnexpectedEos; self.offset, self.buf_len());
@@ -234,7 +235,7 @@ impl<B: AsRef<[u8]> + AsMut<[u8]>> Decode for BytesDecoder<B> {
         }
     }
 
-    fn has_terminated(&self) -> bool {
+    fn is_idle(&self) -> bool {
         self.bytes.is_none()
     }
 
@@ -286,8 +287,8 @@ impl Decode for RemainingBytesDecoder {
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        false
+    fn is_idle(&self) -> bool {
+        self.0.is_empty()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -418,8 +419,8 @@ where
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
