@@ -40,8 +40,8 @@ impl Decode for StartDecoderChain {
         track_panic!(ErrorKind::Other)
     }
 
-    fn has_terminated(&self) -> bool {
-        false
+    fn is_idle(&self) -> bool {
+        true
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -69,8 +69,8 @@ where
         self.0.b.decode(buf, eos).map(|(n, i)| (n, i.map(|i| (i,))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.b.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.b.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -90,8 +90,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -111,8 +111,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -132,8 +132,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -153,8 +153,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -174,8 +174,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -195,8 +195,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, t.5, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -217,8 +217,8 @@ where
             .map(|(n, x)| (n, x.map(|(t, i)| (t.0, t.1, t.2, t.3, t.4, t.5, t.6, i))))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -509,15 +509,8 @@ where
     type Item = (A::Item, B::Item);
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
-        let offset = if self.a.item.is_none() {
-            let (size, _always_none) = track!(self.a.decode(buf, eos))?;
-            if self.a.item.is_none() {
-                return Ok((size, None));
-            }
-            size
-        } else {
-            0
-        };
+        let mut offset = 0;
+        bytecodec_try_decode!(self.a, offset, buf, eos);
 
         let (size, item) = track!(self.b.decode(&buf[offset..], eos))?;
         let item = item.map(|b| {
@@ -527,8 +520,8 @@ where
         Ok((offset + size, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.a.has_terminated() || self.b.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.a.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -546,12 +539,8 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.a.is_idle() {
-            offset += track!(self.a.encode(buf, eos))?;
-        }
-        if self.a.is_idle() {
-            offset += track!(self.b.encode(&mut buf[offset..], eos))?;
-        }
+        bytecodec_try_encode!(self.a, offset, buf, eos);
+        bytecodec_try_encode!(self.b, offset, buf, eos);
         Ok(offset)
     }
 
@@ -656,12 +645,8 @@ impl<D: Decode> Decode for Buffered<D> {
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        if self.item.is_some() {
-            false
-        } else {
-            self.inner.has_terminated()
-        }
+    fn is_idle(&self) -> bool {
+        self.item.is_none() && self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
