@@ -44,8 +44,8 @@ where
         track!(self.inner.decode(buf, eos)).map(|(n, r)| (n, r.map(&self.map)))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -89,8 +89,8 @@ where
             .map_err(|e| (self.map_err)(e).into())
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -192,12 +192,8 @@ where
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        if let Some(ref d) = self.inner1 {
-            d.has_terminated()
-        } else {
-            self.inner0.has_terminated()
-        }
+    fn is_idle(&self) -> bool {
+        self.inner0.is_idle() && self.inner1.is_none()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -380,10 +376,6 @@ impl<E: Default, I> Default for Repeat<E, I> {
     }
 }
 
-/// An alias of `Omittable`.
-#[deprecated(since = "0.2.16", note = "please use `Omittable` instead")]
-pub type Omit<D> = Omittable<D>;
-
 /// Combinator for representing optional decoders.
 ///
 /// This is created by calling `DecodeExt::omit` method.
@@ -437,12 +429,8 @@ impl<D: Decode> Decode for Omittable<D> {
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        if self.do_omit {
-            false
-        } else {
-            self.inner.has_terminated()
-        }
+    fn is_idle(&self) -> bool {
+        self.do_omit || self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -492,9 +480,10 @@ impl<E: ExactBytesEncode> ExactBytesEncode for Optional<E> {
 
 /// Combinator for collecting decoded items.
 ///
-/// This is created by calling `DecodeExt::collect` method.
+/// `Collect` decodes all items until it reaches EOS
+/// and returns the collected items as the single decoded item.
 ///
-/// Note that this is a oneshot decoder (i.e., it decodes only one item).
+/// This is created by calling `DecodeExt::collect` method.
 #[derive(Debug, Default)]
 pub struct Collect<D, T> {
     inner: D,
@@ -516,18 +505,20 @@ where
         if self.items.is_none() {
             self.items = Some(T::default());
         }
-        if (buf.is_empty() && eos.is_reached()) || self.inner.has_terminated() {
+
+        if buf.is_empty() && eos.is_reached() {
             return Ok((0, self.items.take()));
         }
 
-        let items = self.items.as_mut().expect("Never fails");
         let (size, item) = track!(self.inner.decode(buf, eos))?;
-        items.extend(item);
+        if item.is_some() {
+            self.items.as_mut().map(|x| x.extend(item));
+        }
         Ok((size, None))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.items.is_none()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -615,24 +606,18 @@ impl<D: Decode> Decode for Length<D> {
                 "Decoder consumes too few bytes"
             );
             self.remaining_bytes = self.expected_bytes
+        } else {
+            track_assert_ne!(self.remaining_bytes, 0, ErrorKind::Other);
         }
         Ok((size, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        if self.remaining_bytes == self.expected_bytes {
-            self.inner.has_terminated()
-        } else {
-            false
-        }
+    fn is_idle(&self) -> bool {
+        self.remaining_bytes == self.expected_bytes
     }
 
     fn requiring_bytes(&self) -> ByteCount {
-        if self.has_terminated() {
-            ByteCount::Finite(0)
-        } else {
-            ByteCount::Finite(self.remaining_bytes)
-        }
+        ByteCount::Finite(self.remaining_bytes)
     }
 }
 impl<E: Encode> Encode for Length<E> {
@@ -685,32 +670,24 @@ impl<E: Encode> ExactBytesEncode for Length<E> {
     }
 }
 
-/// Combinator for decoding the specified number of items.
+/// Combinator for decoding the specified number of items and collecting the result.
 ///
-/// This is created by calling `DecodeExt::take` method.
+/// This is created by calling `DecodeExt::collectn` method.
 #[derive(Debug, Default)]
-pub struct Take<D> {
+pub struct CollectN<D, T> {
     inner: D,
-    limit: usize,
-    decoded_items: usize,
+    remaining_items: usize,
+    items: Option<T>,
 }
-impl<D> Take<D> {
-    /// Returns the maximum number of items allowed to be decoded.
-    pub fn limit(&self) -> usize {
-        self.limit
+impl<D, T> CollectN<D, T> {
+    /// Returns the number of remaining items expected to be decoded.
+    pub fn remaining_items(&self) -> usize {
+        self.remaining_items
     }
 
-    /// Resets the maximum number of items allowed to be decoded.
-    ///
-    /// The count of items have been decoded so far also will be reset to `0`.
-    pub fn reset_limit(&mut self, n: usize) {
-        self.limit = n;
-        self.decoded_items = 0;
-    }
-
-    /// Returns the number of items have been decoded.
-    pub fn decoded_items(&self) -> usize {
-        self.decoded_items
+    /// Sets the number of remaining items expected to be decoded.
+    pub fn set_remaining_items(&mut self, n: usize) {
+        self.remaining_items = n;
     }
 
     /// Returns a reference to the inner decoder.
@@ -729,33 +706,45 @@ impl<D> Take<D> {
     }
 
     pub(crate) fn new(inner: D, count: usize) -> Self {
-        Take {
+        CollectN {
             inner,
-            limit: count,
-            decoded_items: 0,
+            remaining_items: count,
+            items: None,
         }
     }
 }
-impl<D: Decode> Decode for Take<D> {
-    type Item = D::Item;
+impl<D, T> Decode for CollectN<D, T>
+where
+    D: Decode,
+    T: Default + Extend<D::Item>,
+{
+    type Item = T;
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
-        track_assert_ne!(self.decoded_items, self.limit, ErrorKind::DecoderTerminated);
-        match track!(self.inner.decode(buf, eos))? {
-            (size, Some(item)) => {
-                self.decoded_items += 1;
-                Ok((size, Some(item)))
+        if self.items.is_none() {
+            if self.remaining_items == 0 {
+                return Ok((0, Some(T::default())));
             }
-            (size, None) => Ok((size, None)),
+            self.items = Some(T::default());
         }
+
+        let (size, item) = track!(self.inner.decode(buf, eos))?;
+        if item.is_some() {
+            self.items.as_mut().map(|x| x.extend(item));
+            self.remaining_items -= 1;
+            if self.remaining_items == 0 {
+                return Ok((size, self.items.take()));
+            }
+        }
+        Ok((size, None))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated() || self.decoded_items == self.limit
+    fn is_idle(&self) -> bool {
+        self.items.is_none()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
-        if self.has_terminated() {
+        if self.remaining_items == 0 {
             ByteCount::Finite(0)
         } else {
             self.inner.requiring_bytes()
@@ -799,8 +788,8 @@ where
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -856,12 +845,8 @@ impl<D: Decode> Decode for SkipRemaining<D> {
         }
     }
 
-    fn has_terminated(&self) -> bool {
-        if self.item.is_none() {
-            self.inner.has_terminated()
-        } else {
-            false
-        }
+    fn is_idle(&self) -> bool {
+        self.item.is_none() && self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -876,6 +861,8 @@ impl<D: Decode> Decode for SkipRemaining<D> {
 /// Combinator that will fail if the number of consumed bytes exceeds the specified size.
 ///
 /// This is created by calling `{DecodeExt, EncodeExt}::max_bytes` method.
+///
+/// Note that `MaxBytes` assumes the inner decoder will consume all the bytes in the target stream.
 #[derive(Debug, Default)]
 pub struct MaxBytes<C> {
     inner: C,
@@ -902,8 +889,19 @@ impl<C> MaxBytes<C> {
     }
 
     /// Sets the maximum number of bytes that can be consumed for encoding/decoding an item.
-    pub fn set_max_bytes(&mut self, n: u64) {
+    ///
+    /// # Error
+    ///
+    /// If `n` is smaller than `self.consumed_bytes()`, an `ErrorKind::InvalidInput` error will be returned.
+    pub fn set_max_bytes(&mut self, n: u64) -> Result<()> {
+        track_assert!(
+            self.consumed_bytes <= n,
+            ErrorKind::InvalidInput;
+            self.consumed_bytes,
+            n
+        );
         self.max_bytes = n;
+        Ok(())
     }
 
     /// Returns a reference to the inner encoder or decoder.
@@ -925,19 +923,35 @@ impl<D: Decode> Decode for MaxBytes<D> {
     type Item = D::Item;
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
+        match eos.remaining_bytes() {
+            ByteCount::Infinite => {
+                track_panic!(ErrorKind::InvalidInput, "Max bytes limit exceeded";
+                             self.consumed_bytes, self.max_bytes)
+            }
+            ByteCount::Unknown => {
+                let consumable_bytes = self.max_bytes - self.consumed_bytes;
+                track_assert!((buf.len() as u64) <= consumable_bytes,
+                              ErrorKind::InvalidInput, "Max bytes limit exceeded";
+                              buf.len(), self.consumed_bytes, self.max_bytes);
+            }
+            ByteCount::Finite(remaining_bytes) => {
+                let consumable_bytes = self.max_bytes - self.consumed_bytes;
+                track_assert!((buf.len() as u64) + remaining_bytes <= consumable_bytes,
+                              ErrorKind::InvalidInput, "Max bytes limit exceeded";
+                              buf.len(), remaining_bytes, self.consumed_bytes, self.max_bytes)
+            }
+        }
+
         let (size, item) = track!(self.inner.decode(buf, eos))?;
         self.consumed_bytes += size as u64;
-        track_assert!(self.consumed_bytes <= self.max_bytes,
-                      ErrorKind::InvalidInput, "Max bytes limit exceeded";
-                      self.consumed_bytes, self.max_bytes);
         if item.is_some() {
             self.consumed_bytes = 0;
         }
         Ok((size, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -1004,8 +1018,8 @@ where
         Ok((size, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -1051,15 +1065,14 @@ impl<E: Encode> Encode for Padding<E> {
     type Item = E::Item;
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
-        if !self.inner.is_idle() {
-            self.inner.encode(buf, eos)
-        } else {
-            for b in buf.iter_mut() {
-                *b = self.padding_byte;
-            }
-            self.eos_reached = eos.is_reached();
-            Ok(buf.len())
+        let mut offset = 0;
+        bytecodec_try_encode!(self.inner, offset, buf, eos);
+
+        for b in (&mut buf[offset..]).iter_mut() {
+            *b = self.padding_byte;
         }
+        self.eos_reached = eos.is_reached();
+        Ok(buf.len())
     }
 
     fn start_encoding(&mut self, item: Self::Item) -> Result<()> {
@@ -1104,11 +1117,10 @@ where
     type Item = E0::Item;
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
-        if !self.prefix_encoder.is_idle() {
-            track!(self.prefix_encoder.encode(buf, eos))
-        } else {
-            track!(self.body_encoder.encode(buf, eos))
-        }
+        let mut offset = 0;
+        bytecodec_try_encode!(self.prefix_encoder, offset, buf, eos);
+        bytecodec_try_encode!(self.body_encoder, offset, buf, eos);
+        Ok(offset)
     }
 
     fn start_encoding(&mut self, item: Self::Item) -> Result<()> {
@@ -1251,8 +1263,8 @@ impl<D: Decode> Decode for Slice<D> {
         Ok((size, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.inner.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -1400,69 +1412,12 @@ impl<E: Encode> Encode for LastItem<E> {
     }
 }
 
-/// Combinator for allowing EOS reached before decoding starts.
-#[derive(Debug, Default)]
-pub struct MaybeEos<D> {
-    inner: D,
-    started: bool,
-}
-impl<D> MaybeEos<D> {
-    /// Makes a new `MaybeEos` instance.
-    pub fn new(inner: D) -> Self {
-        MaybeEos {
-            inner,
-            started: false,
-        }
-    }
-
-    /// Returns a reference to the inner decoder.
-    pub fn inner_ref(&self) -> &D {
-        &self.inner
-    }
-
-    /// Returns a mutable reference to the inner decoder.
-    pub fn inner_mut(&mut self) -> &mut D {
-        &mut self.inner
-    }
-
-    /// Takes ownership of this instance and returns the inner decoder.
-    pub fn into_inner(self) -> D {
-        self.inner
-    }
-}
-impl<D: Decode> Decode for MaybeEos<D> {
-    type Item = D::Item;
-
-    fn decode(&mut self, buf: &[u8], mut eos: Eos) -> Result<(usize, Option<Self::Item>)> {
-        if !self.started && eos.is_reached() {
-            eos = Eos::new(false);
-        }
-        let (size, item) = track!(self.inner.decode(buf, eos))?;
-        if let Some(item) = item {
-            self.started = false;
-            Ok((size, Some(item)))
-        } else {
-            self.started |= size > 0;
-            Ok((size, None))
-        }
-    }
-
-    fn has_terminated(&self) -> bool {
-        self.inner.has_terminated()
-    }
-
-    fn requiring_bytes(&self) -> ByteCount {
-        self.inner.requiring_bytes()
-    }
-}
-
 #[cfg(test)]
 mod test {
     use {Decode, DecodeExt, Encode, EncodeExt, Eos, ErrorKind};
     use bytes::{Utf8Decoder, Utf8Encoder};
-    use fixnum::{U16beDecoder, U8Decoder, U8Encoder};
+    use fixnum::{U8Decoder, U8Encoder};
     use io::{IoDecodeExt, IoEncodeExt};
-    use super::*;
 
     #[test]
     fn collect_works() {
@@ -1472,10 +1427,19 @@ mod test {
     }
 
     #[test]
-    fn take_works() {
-        let mut decoder = U8Decoder::new().take(2).collect::<Vec<_>>();
+    fn collectn_works() {
+        let mut decoder = U8Decoder::new().collectn::<Vec<_>>(2);
         let item = track_try_unwrap!(decoder.decode_exact(b"foo".as_ref()));
         assert_eq!(item, vec![b'f', b'o']);
+
+        let mut decoder = U8Decoder::new().collectn::<Vec<_>>(4);
+        assert_eq!(
+            decoder
+                .decode_exact(b"foo".as_ref())
+                .err()
+                .map(|e| *e.kind()),
+            Some(ErrorKind::UnexpectedEos)
+        );
     }
 
     #[test]
@@ -1611,17 +1575,4 @@ mod test {
         let (_, item) = track_try_unwrap!(decoder.decode(b"\x03foo", Eos::new(false)));
         assert_eq!(item, Some("foo".to_owned()));
     }
-
-    #[test]
-    fn maybe_eos_works() {
-        let mut decoder = U16beDecoder::new();
-        assert!(decoder.decode(&[][..], Eos::new(true)).is_err());
-
-        let mut decoder = MaybeEos::new(U16beDecoder::new());
-        assert!(decoder.decode(&[][..], Eos::new(true)).is_ok());
-
-        let mut decoder = MaybeEos::new(U16beDecoder::new());
-        assert!(decoder.decode(&[1][..], Eos::new(false)).is_ok());
-        assert!(decoder.decode(&[][..], Eos::new(true)).is_err());
-    }
 }
