@@ -1,8 +1,8 @@
 use std;
 
 use {ByteCount, Eos, Error, Result};
-use combinator::{AndThen, Assert, Buffered, Collect, DecoderChain, Length, Map, MapErr, MaxBytes,
-                 Omittable, SkipRemaining, Slice, Take, TryMap};
+use combinator::{AndThen, Assert, Buffered, Collect, CollectN, DecoderChain, Length, Map, MapErr,
+                 MaxBytes, Omittable, SkipRemaining, Slice, TryMap};
 
 /// This trait allows for decoding items from a byte sequence incrementally.
 pub trait Decode {
@@ -41,11 +41,8 @@ pub trait Decode {
     ///   - Other errors
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)>;
 
-    /// Returns `true` if the decoder cannot decode items anymore, otherwise `false`.
-    ///
-    /// If it returns `true`, the next invocation of `decode` method
-    /// must return an `ErrorKind::DecoderTerminated` error, and vice versa.
-    fn has_terminated(&self) -> bool;
+    /// Returns `true` if there is no item being decoded by the decoder, otherwise `false`.
+    fn is_idle(&self) -> bool;
 
     /// Returns the lower bound of the number of bytes needed to decode the next item.
     ///
@@ -55,8 +52,10 @@ pub trait Decode {
     /// If the decoder returns `ByteCount::Finite(0)`, it means one of the followings:
     /// - (a) There is an already decoded item
     ///   - The next invocation of `decode()` will return it without consuming any bytes
+    ///   - In this case, the invocation of `Decode::is_idle` method must return `false`.
     /// - (b) There are no decodable items
     ///   - All decodable items have been decoded, and the decoder cannot do any further works
+    ///   - In this case, the invocation of `Decode::is_idle` method must return `true`.
     fn requiring_bytes(&self) -> ByteCount;
 }
 impl<'a, D: ?Sized + Decode> Decode for &'a mut D {
@@ -66,8 +65,8 @@ impl<'a, D: ?Sized + Decode> Decode for &'a mut D {
         (**self).decode(buf, eos)
     }
 
-    fn has_terminated(&self) -> bool {
-        (**self).has_terminated()
+    fn is_idle(&self) -> bool {
+        (**self).is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -81,8 +80,8 @@ impl<D: ?Sized + Decode> Decode for Box<D> {
         (**self).decode(buf, eos)
     }
 
-    fn has_terminated(&self) -> bool {
-        (**self).has_terminated()
+    fn is_idle(&self) -> bool {
+        (**self).is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -263,6 +262,26 @@ pub trait DecodeExt: Decode + Sized {
         Collect::new(self)
     }
 
+    /// Creates a decoder that decodes `n` items by using `self` and collecting the result.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use bytecodec::{Decode, DecodeExt};
+    /// use bytecodec::fixnum::U8Decoder;
+    /// use bytecodec::io::IoDecodeExt;
+    ///
+    /// let mut decoder = U8Decoder::new().collectn::<Vec<_>>(2);
+    /// let item = decoder.decode_exact(b"foo".as_ref()).unwrap();
+    /// assert_eq!(item, vec![b'f', b'o']);
+    /// ```
+    fn collectn<T>(self, n: usize) -> CollectN<Self, T>
+    where
+        T: Extend<Self::Item> + Default,
+    {
+        CollectN::new(self, n)
+    }
+
     /// Creates a decoder that consumes the specified number of bytes exactly.
     ///
     /// # Examples
@@ -288,23 +307,6 @@ pub trait DecodeExt: Decode + Sized {
         Length::new(self, expected_bytes)
     }
 
-    /// Creates a decoder that decodes `n` items by using `self`.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use bytecodec::{Decode, DecodeExt};
-    /// use bytecodec::fixnum::U8Decoder;
-    /// use bytecodec::io::IoDecodeExt;
-    ///
-    /// let mut decoder = U8Decoder::new().take(2).collect::<Vec<_>>();
-    /// let item = decoder.decode_exact(b"foo".as_ref()).unwrap();
-    /// assert_eq!(item, vec![b'f', b'o']);
-    /// ```
-    fn take(self, n: usize) -> Take<Self> {
-        Take::new(self, n)
-    }
-
     /// Creates a decoder that will omit decoding items if `do_omit = true` is specified.
     ///
     /// # Examples
