@@ -14,8 +14,8 @@ macro_rules! impl_decode {
                 Ok((size, item.map(Self::decode_item)))
             }
 
-            fn has_terminated(&self) -> bool {
-                false
+            fn is_idle(&self) -> bool {
+                self.0.is_idle()
             }
 
             fn requiring_bytes(&self) -> ByteCount {
@@ -47,10 +47,6 @@ macro_rules! impl_encode {
             fn is_idle(&self) -> bool {
                 self.0.is_idle()
             }
-
-            fn cancel(&mut self) -> Result<()> {
-                track!(self.0.cancel())
-            }
         }
         impl ExactBytesEncode for $ty {
             fn exact_requiring_bytes(&self) -> u64 {
