@@ -55,8 +55,8 @@ where
         track!(self.0.decode(buf, eos))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.0.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
