@@ -45,10 +45,7 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
 
         let (size, item) = track!(self.d1.decode(&buf[offset..], eos))?;
         offset += size;
@@ -57,8 +54,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -115,15 +112,8 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d1.decode(&buf[offset..], eos))?.0;
-        if !self.d1.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
+        bytecodec_try_decode!(self.d1, offset, buf, eos);
 
         let (size, item) = track!(self.d2.decode(&buf[offset..], eos))?;
         offset += size;
@@ -136,8 +126,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated() || self.d2.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -210,20 +200,9 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d1.decode(&buf[offset..], eos))?.0;
-        if !self.d1.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d2.decode(&buf[offset..], eos))?.0;
-        if !self.d2.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
+        bytecodec_try_decode!(self.d1, offset, buf, eos);
+        bytecodec_try_decode!(self.d2, offset, buf, eos);
 
         let (size, item) = track!(self.d3.decode(&buf[offset..], eos))?;
         offset += size;
@@ -237,9 +216,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated() || self.d2.has_terminated()
-            || self.d3.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -320,25 +298,10 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d1.decode(&buf[offset..], eos))?.0;
-        if !self.d1.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d2.decode(&buf[offset..], eos))?.0;
-        if !self.d2.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d3.decode(&buf[offset..], eos))?.0;
-        if !self.d3.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
+        bytecodec_try_decode!(self.d1, offset, buf, eos);
+        bytecodec_try_decode!(self.d2, offset, buf, eos);
+        bytecodec_try_decode!(self.d3, offset, buf, eos);
 
         let (size, item) = track!(self.d4.decode(&buf[offset..], eos))?;
         offset += size;
@@ -353,9 +316,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated() || self.d2.has_terminated()
-            || self.d3.has_terminated() || self.d4.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -444,30 +406,11 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d1.decode(&buf[offset..], eos))?.0;
-        if !self.d1.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d2.decode(&buf[offset..], eos))?.0;
-        if !self.d2.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d3.decode(&buf[offset..], eos))?.0;
-        if !self.d3.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d4.decode(&buf[offset..], eos))?.0;
-        if !self.d4.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
+        bytecodec_try_decode!(self.d1, offset, buf, eos);
+        bytecodec_try_decode!(self.d2, offset, buf, eos);
+        bytecodec_try_decode!(self.d3, offset, buf, eos);
+        bytecodec_try_decode!(self.d4, offset, buf, eos);
 
         let (size, item) = track!(self.d5.decode(&buf[offset..], eos))?;
         offset += size;
@@ -483,10 +426,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated() || self.d2.has_terminated()
-            || self.d3.has_terminated() || self.d4.has_terminated()
-            || self.d5.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -601,35 +542,12 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d1.decode(&buf[offset..], eos))?.0;
-        if !self.d1.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d2.decode(&buf[offset..], eos))?.0;
-        if !self.d2.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d3.decode(&buf[offset..], eos))?.0;
-        if !self.d3.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d4.decode(&buf[offset..], eos))?.0;
-        if !self.d4.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d5.decode(&buf[offset..], eos))?.0;
-        if !self.d5.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
+        bytecodec_try_decode!(self.d1, offset, buf, eos);
+        bytecodec_try_decode!(self.d2, offset, buf, eos);
+        bytecodec_try_decode!(self.d3, offset, buf, eos);
+        bytecodec_try_decode!(self.d4, offset, buf, eos);
+        bytecodec_try_decode!(self.d5, offset, buf, eos);
 
         let (size, item) = track!(self.d6.decode(&buf[offset..], eos))?;
         offset += size;
@@ -646,10 +564,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated() || self.d2.has_terminated()
-            || self.d3.has_terminated() || self.d4.has_terminated()
-            || self.d5.has_terminated() || self.d6.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -774,40 +690,13 @@ where
 
     fn decode(&mut self, buf: &[u8], eos: Eos) -> Result<(usize, Option<Self::Item>)> {
         let mut offset = 0;
-        offset += track!(self.d0.decode(buf, eos))?.0;
-        if !self.d0.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d1.decode(&buf[offset..], eos))?.0;
-        if !self.d1.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d2.decode(&buf[offset..], eos))?.0;
-        if !self.d2.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d3.decode(&buf[offset..], eos))?.0;
-        if !self.d3.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d4.decode(&buf[offset..], eos))?.0;
-        if !self.d4.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d5.decode(&buf[offset..], eos))?.0;
-        if !self.d5.has_item() {
-            return Ok((offset, None));
-        }
-
-        offset += track!(self.d6.decode(&buf[offset..], eos))?.0;
-        if !self.d6.has_item() {
-            return Ok((offset, None));
-        }
+        bytecodec_try_decode!(self.d0, offset, buf, eos);
+        bytecodec_try_decode!(self.d1, offset, buf, eos);
+        bytecodec_try_decode!(self.d2, offset, buf, eos);
+        bytecodec_try_decode!(self.d3, offset, buf, eos);
+        bytecodec_try_decode!(self.d4, offset, buf, eos);
+        bytecodec_try_decode!(self.d5, offset, buf, eos);
+        bytecodec_try_decode!(self.d6, offset, buf, eos);
 
         let (size, item) = track!(self.d7.decode(&buf[offset..], eos))?;
         offset += size;
@@ -825,11 +714,8 @@ where
         Ok((offset, item))
     }
 
-    fn has_terminated(&self) -> bool {
-        self.d0.has_terminated() || self.d1.has_terminated() || self.d2.has_terminated()
-            || self.d3.has_terminated() || self.d4.has_terminated()
-            || self.d5.has_terminated() || self.d6.has_terminated()
-            || self.d7.has_terminated()
+    fn is_idle(&self) -> bool {
+        self.d0.is_idle()
     }
 
     fn requiring_bytes(&self) -> ByteCount {
@@ -880,13 +766,8 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
         Ok(offset)
     }
 
@@ -905,12 +786,6 @@ where
     fn is_idle(&self) -> bool {
         self.e0.is_idle() && self.e1.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1> ExactBytesEncode for Tuple2Encoder<E0, E1>
 where
@@ -960,19 +835,9 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e1.is_idle() {
-            offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
-            if !self.e1.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e2.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
+        bytecodec_try_encode!(self.e2, offset, buf, eos);
         Ok(offset)
     }
 
@@ -993,13 +858,6 @@ where
     fn is_idle(&self) -> bool {
         self.e0.is_idle() && self.e1.is_idle() && self.e2.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        track!(self.e2.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1, E2> ExactBytesEncode for Tuple3Encoder<E0, E1, E2>
 where
@@ -1054,25 +912,10 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e1.is_idle() {
-            offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
-            if !self.e1.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e2.is_idle() {
-            offset += track!(self.e2.encode(&mut buf[offset..], eos))?;
-            if !self.e2.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e3.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
+        bytecodec_try_encode!(self.e2, offset, buf, eos);
+        bytecodec_try_encode!(self.e3, offset, buf, eos);
         Ok(offset)
     }
 
@@ -1095,14 +938,6 @@ where
     fn is_idle(&self) -> bool {
         self.e0.is_idle() && self.e1.is_idle() && self.e2.is_idle() && self.e3.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        track!(self.e2.cancel())?;
-        track!(self.e3.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1, E2, E3> ExactBytesEncode for Tuple4Encoder<E0, E1, E2, E3>
 where
@@ -1167,31 +1002,11 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e1.is_idle() {
-            offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
-            if !self.e1.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e2.is_idle() {
-            offset += track!(self.e2.encode(&mut buf[offset..], eos))?;
-            if !self.e2.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e3.is_idle() {
-            offset += track!(self.e3.encode(&mut buf[offset..], eos))?;
-            if !self.e3.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e4.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
+        bytecodec_try_encode!(self.e2, offset, buf, eos);
+        bytecodec_try_encode!(self.e3, offset, buf, eos);
+        bytecodec_try_encode!(self.e4, offset, buf, eos);
         Ok(offset)
     }
 
@@ -1217,15 +1032,6 @@ where
         self.e0.is_idle() && self.e1.is_idle() && self.e2.is_idle() && self.e3.is_idle()
             && self.e4.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        track!(self.e2.cancel())?;
-        track!(self.e3.cancel())?;
-        track!(self.e4.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1, E2, E3, E4> ExactBytesEncode for Tuple5Encoder<E0, E1, E2, E3, E4>
 where
@@ -1303,37 +1109,12 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e1.is_idle() {
-            offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
-            if !self.e1.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e2.is_idle() {
-            offset += track!(self.e2.encode(&mut buf[offset..], eos))?;
-            if !self.e2.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e3.is_idle() {
-            offset += track!(self.e3.encode(&mut buf[offset..], eos))?;
-            if !self.e3.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e4.is_idle() {
-            offset += track!(self.e4.encode(&mut buf[offset..], eos))?;
-            if !self.e4.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e5.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
+        bytecodec_try_encode!(self.e2, offset, buf, eos);
+        bytecodec_try_encode!(self.e3, offset, buf, eos);
+        bytecodec_try_encode!(self.e4, offset, buf, eos);
+        bytecodec_try_encode!(self.e5, offset, buf, eos);
         Ok(offset)
     }
 
@@ -1361,16 +1142,6 @@ where
         self.e0.is_idle() && self.e1.is_idle() && self.e2.is_idle() && self.e3.is_idle()
             && self.e4.is_idle() && self.e5.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        track!(self.e2.cancel())?;
-        track!(self.e3.cancel())?;
-        track!(self.e4.cancel())?;
-        track!(self.e5.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1, E2, E3, E4, E5> ExactBytesEncode for Tuple6Encoder<E0, E1, E2, E3, E4, E5>
 where
@@ -1480,43 +1251,13 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e1.is_idle() {
-            offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
-            if !self.e1.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e2.is_idle() {
-            offset += track!(self.e2.encode(&mut buf[offset..], eos))?;
-            if !self.e2.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e3.is_idle() {
-            offset += track!(self.e3.encode(&mut buf[offset..], eos))?;
-            if !self.e3.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e4.is_idle() {
-            offset += track!(self.e4.encode(&mut buf[offset..], eos))?;
-            if !self.e4.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e5.is_idle() {
-            offset += track!(self.e5.encode(&mut buf[offset..], eos))?;
-            if !self.e5.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e6.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
+        bytecodec_try_encode!(self.e2, offset, buf, eos);
+        bytecodec_try_encode!(self.e3, offset, buf, eos);
+        bytecodec_try_encode!(self.e4, offset, buf, eos);
+        bytecodec_try_encode!(self.e5, offset, buf, eos);
+        bytecodec_try_encode!(self.e6, offset, buf, eos);
         Ok(offset)
     }
 
@@ -1546,17 +1287,6 @@ where
         self.e0.is_idle() && self.e1.is_idle() && self.e2.is_idle() && self.e3.is_idle()
             && self.e4.is_idle() && self.e5.is_idle() && self.e6.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        track!(self.e2.cancel())?;
-        track!(self.e3.cancel())?;
-        track!(self.e4.cancel())?;
-        track!(self.e5.cancel())?;
-        track!(self.e6.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1, E2, E3, E4, E5, E6> ExactBytesEncode for Tuple7Encoder<E0, E1, E2, E3, E4, E5, E6>
 where
@@ -1676,49 +1406,14 @@ where
 
     fn encode(&mut self, buf: &mut [u8], eos: Eos) -> Result<usize> {
         let mut offset = 0;
-        if !self.e0.is_idle() {
-            offset += track!(self.e0.encode(buf, eos))?;
-            if !self.e0.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e1.is_idle() {
-            offset += track!(self.e1.encode(&mut buf[offset..], eos))?;
-            if !self.e1.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e2.is_idle() {
-            offset += track!(self.e2.encode(&mut buf[offset..], eos))?;
-            if !self.e2.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e3.is_idle() {
-            offset += track!(self.e3.encode(&mut buf[offset..], eos))?;
-            if !self.e3.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e4.is_idle() {
-            offset += track!(self.e4.encode(&mut buf[offset..], eos))?;
-            if !self.e4.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e5.is_idle() {
-            offset += track!(self.e5.encode(&mut buf[offset..], eos))?;
-            if !self.e5.is_idle() {
-                return Ok(offset);
-            }
-        }
-        if !self.e6.is_idle() {
-            offset += track!(self.e6.encode(&mut buf[offset..], eos))?;
-            if !self.e6.is_idle() {
-                return Ok(offset);
-            }
-        }
-        offset += track!(self.e7.encode(&mut buf[offset..], eos))?;
+        bytecodec_try_encode!(self.e0, offset, buf, eos);
+        bytecodec_try_encode!(self.e1, offset, buf, eos);
+        bytecodec_try_encode!(self.e2, offset, buf, eos);
+        bytecodec_try_encode!(self.e3, offset, buf, eos);
+        bytecodec_try_encode!(self.e4, offset, buf, eos);
+        bytecodec_try_encode!(self.e5, offset, buf, eos);
+        bytecodec_try_encode!(self.e6, offset, buf, eos);
+        bytecodec_try_encode!(self.e7, offset, buf, eos);
         Ok(offset)
     }
 
@@ -1751,18 +1446,6 @@ where
             && self.e4.is_idle() && self.e5.is_idle() && self.e6.is_idle()
             && self.e7.is_idle()
     }
-
-    fn cancel(&mut self) -> Result<()> {
-        track!(self.e0.cancel())?;
-        track!(self.e1.cancel())?;
-        track!(self.e2.cancel())?;
-        track!(self.e3.cancel())?;
-        track!(self.e4.cancel())?;
-        track!(self.e5.cancel())?;
-        track!(self.e6.cancel())?;
-        track!(self.e7.cancel())?;
-        Ok(())
-    }
 }
 impl<E0, E1, E2, E3, E4, E5, E6, E7> ExactBytesEncode
     for Tuple8Encoder<E0, E1, E2, E3, E4, E5, E6, E7>
