@@ -33,7 +33,7 @@ impl<T> Decode for DecodedValue<T> {
         Ok((0, Some(item)))
     }
 
-    fn has_terminated(&self) -> bool {
+    fn is_idle(&self) -> bool {
         self.0.is_none()
     }
 
@@ -52,7 +52,7 @@ impl Decode for NullDecoder {
         Ok((0, Some(())))
     }
 
-    fn has_terminated(&self) -> bool {
+    fn is_idle(&self) -> bool {
         false
     }
 
